{*************************************************************************
  Procedimentos para o envio de SMSs atravs do servio
  SMS Facil
  www.smsfacil.com

  Autor: George De Luca
  Data: 25-02-2011
*************************************************************************}
unit uSMSFacil;

interface

Uses
  Classes, SysUtils,
  IniFiles,  // para leitura do INI
  IdHTTP;    // para utilizao do componetne do INDY

Type
  regSMS = Record   // record para tratar a mensagem a ser enviada
    Telefone,
    Mensagem,
    Login,
    Senha,
    From : String;
    ComErro : Boolean;

    // funo para "traduzir" o erro que  retornado pelo SMS Fcil
    Function  TraduzErro(AErro : String):String;
    // funo padro para setar o texto da mensagem a ser enviada
    Function  setMensagem(AMens : String; Var AErro : String) : Boolean;
    // funo padro para setar o telefone do destinatrio
    Function  setTelefone(ATel : String; Var AErro : String) : Boolean;
    // funo padro para setar enviar o SMS
    Function  EnviaSMS : String;
  End;

implementation

uses uLog;

{ regSMS }

Function regSMS.EnviaSMS : String;
Var
  IniFile : TIniFile;
  sTipo, sRetorno, sURL : String;
  idHTTP : TidHTTP;
begin
  ComErro := False;
  // verifico se o arquivo INI com os parametros existe
  if not FileExists('EnviaSMS.ini') then
  Begin
    // se no existir, retornar erro
    ComErro := True;
    Result  := 'Arquivo EnviaSMS.ini no existe.';
  End
  Else
  Begin
    // leio o arquivo INI
    IniFile := TIniFile.Create('.\EnviaSMS.ini'); // uses IniFiles
    // tipo do servio
    sTipo := IniFile.ReadString('GERAL','TIPO','erro');
    // login no SMS Fcil
    Login := IniFile.ReadString('LOGIN','LOGIN','erro');
    // Senha no SMS Fcil
    Senha := IniFile.ReadString('LOGIN','SENHA','erro');
    // telefone vlido para enviar as mensagens
    From  := IniFile.ReadString('LOGIN','FROM','erro');
    // sRetorno neste momento conter a soma dos contedos das variveis acima
    // caso alguma tenha retornado a string "erro" ser porqu no existe a varivel
    // no arquivo INI
    sRetorno := sTipo+Login+Senha+From;
    if pos('erro',sRetorno) > 0 Then
    Begin
      // aguma das variveis do INI no existe
      ComErro := True;
      Result  := 'Erro no EnviaSMS.ini. Parmetros invlidos.';
    End
    Else if Trim(sTipo) <> 'SMSFACIL' then
    Begin
      // se o arquivo INI no estiver setado para o SMS Fcil
      ComErro := True;
      Result := 'Tipo de arquivo INI invlido.';
    End
    Else
    Begin
      Try
        // inicio o envio do SMS
        // uses IdHTTP
        idHTTP := TidHTTP.Create(nil); // este componente que ir enviar o SMS
                                       // atravs de HTTP
        sURL :=
          Format(
          // URL utilizada para enviar o SMS por HTTP
'http://www.smsfacil.com.br/api/EnviaSMS/?usuario=%s&senha=%s&de=%s&para=%s&texto=%s',
          [Login,Senha,From,Telefone,Mensagem]);
        // envia o SMS
        sRetorno  := idHTTP.Get(sURL);
        // neste momento, sRetorno conter o texto que foi processado
        // pelo retorno de idHTTP

        // Ao enviar o SMS, ocorrer algum erro, o tamanho do retorno ser de <= 2
        // veja a tabela de erros consultando www.smsfacil.com.br
        if Length(trim(sRetorno)) <= 2 then
        Begin
          // ocorreu erro ao enviar o SMS
          ComErro := True;
          Result := TraduzErro(sRetorno)
        End
        Else
        Begin
          // com sucesso!
          Result := sRetorno;
        End;
      Except
        On E : Exception Do
        Begin
          ComErro := True;
          Result := 'regSMS.EnviaSMS: '+e.Message;
        End;
      End;
    End;
  End;
end;

Function regSMS.setMensagem(AMens: String; Var AErro : String) : Boolean;
// guardo o texto da mensagem na varivel "Mensagem"
// ***Parametros:
// AMens -> o texto da mensagem a ser envida
// AErro -> caso ocorra erros, a mensagem de erro ser guardada neste parmetro
begin
  AErro := '';
  // se o texto da mensagem for maior que 160 caracteres...
  If Length(Trim(AMens)) > 160 Then
  Begin
    // retorna erro que a mensagem  muito extensa
    Result := False;
    AErro  := 'Mensagem extensa. Mximo de 160 caracteres.';
  End
  Else
  Begin
    Result   := True;
    Mensagem := Trim(AMens);
    // substitui os espaos por +
    // ao enviar SMS por HTTP, no  aceito espaos entre as palavras da
    // mensagem, por isso, tem que substituir o espao por "+"
    Mensagem := StringReplace(Mensagem,' ','+',[rfReplaceAll]);
  End;
end;

Function regSMS.setTelefone(ATel: String; Var AErro : String) : Boolean;
// guarda o nmero de celular na varivel "ATel"
// ***Parametros:
// ATel -> o nmero do celular do destinatrio
// AErro -> caso ocorra erros, a mensagem de erro ser guardada neste parmetro
begin
  AErro := '';
  // caso o nmero do celular seja diferente de 10...
  if (Length(ATel) <> 10) then
  Begin
    // retorna erro de nmero de celular invlido
    Result := False;
    AErro  := 'Nmero do celular invlido.';
  End
  Else
  Begin
    Result   := True;
    Telefone := ATel;
  End;
end;

function regSMS.TraduzErro(AErro : String): String;
// traduz os erros do SMS Fcil
begin
  Result := 'Erro inesperado';
  if AErro = '-2' then
    Result := 'Formato de nmero mvel de destinatrio invlido.'
  Else if AErro = '-3' then
    Result := 'Texto da mensagem est vazio.'
  Else if AErro = '-4' then
    Result := 'Nmero mvel invlido ou no suportado pelo sistema.'
  Else if AErro = '-5' then
    Result := 'Sistema indisponvel tente novamente mais tarde.'
  Else if AErro = '-7' then
    Result := 'Crdito insuficiente para realizar o envio.'
  Else if AErro = '-8' then
    Result := 'Usurio ou senha invlidos.'
  Else if AErro = '-9' then
    Result := 'Data ou hora de agendamento invlidos.';
end;

end.
