-- SCRIPT DO BANCO DE DADOS "ActiveDelphi"
-- Autor: Ryan Bruno C Padilha (ryan.padilha@gmail.com)
-- Revista Active Delhi: Orientao a Objetos no Delphi - Srie de 3 artigos.

-- EMPRESA
CREATE TABLE EMPRESA(
  EMP_CODIGO		SERIAL NOT NULL,
  EMP_RAZAO		VARCHAR(50),
  EMP_FANTASIA		VARCHAR(50),
  EMP_DT_CADASTRO	DATE,
  EMP_DT_ABERTURA	DATE,
  EMP_STATUS		INTEGER,
  EMP_ALIAS		VARCHAR(50),
  EMP_TELEFONE		VARCHAR(14),
  EMP_FONEFAX		VARCHAR(14),
  EMP_CNPJ		VARCHAR(20),
  EMP_IE		VARCHAR(20),
  EMP_IEST		VARCHAR(20),
  EMP_IM		VARCHAR(20),
  EMP_CRT		INTEGER,
  EMP_CNAE		VARCHAR(10),
  EMP_EMAIL		VARCHAR(50),
  EMP_PAGINA		VARCHAR(50),
  EMP_CONTATO		VARCHAR(50),
  EMP_MENSAGEM		TEXT
);
ALTER TABLE EMPRESA ADD CONSTRAINT PK_EMPRESA PRIMARY KEY(EMP_CODIGO);

-- ENDERECO
CREATE TABLE ENDERECO(
  END_CODIGO		SERIAL NOT NULL,
  END_LOGRADOURO	VARCHAR(100) NOT NULL,
  END_NUMERO		VARCHAR(10) NOT NULL,
  END_COMPLEMENTO	VARCHAR(50),
  END_CEP		VARCHAR(10),
  END_TIPO		INTEGER,
  END_CORRESP		BOOLEAN,
  END_PT_REFERENCIA	VARCHAR(200),
  END_BAIRRO		VARCHAR(100),
  END_CIDADE		VARCHAR(100)
);
ALTER TABLE ENDERECO ADD CONSTRAINT PK_ENDERECO PRIMARY KEY(END_CODIGO);
ALTER TABLE ENDERECO ADD COLUMN EMP_CODIGO INTEGER;
ALTER TABLE ENDERECO ADD CONSTRAINT FK_END_EMPRESA FOREIGN KEY(EMP_CODIGO) REFERENCES EMPRESA(EMP_CODIGO);

-- MARCA
CREATE TABLE MARCA(
  MAR_CODIGO		SERIAL NOT NULL,
  MAR_DESCRICAO 	VARCHAR(100) NOT NULL
);

ALTER TABLE MARCA ADD CONSTRAINT PK_MARCA PRIMARY KEY(MAR_CODIGO);

INSERT INTO MARCA (MAR_DESCRICAO) VALUES ('DUREX');
INSERT INTO MARCA (MAR_DESCRICAO) VALUES ('EATON');
INSERT INTO MARCA (MAR_DESCRICAO) VALUES ('INDISA');
INSERT INTO MARCA (MAR_DESCRICAO) VALUES ('LUCIFLEX');
INSERT INTO MARCA (MAR_DESCRICAO) VALUES ('PERKINS');
INSERT INTO MARCA (MAR_DESCRICAO) VALUES ('STAHL');

-- UNIDADE
CREATE TABLE UNIDADE(
  UND_CODIGO		SERIAL NOT NULL,
  UND_DESCRICAO		VARCHAR(100) NOT NULL,
  UND_SIGLA		VARCHAR(5) UNIQUE NOT NULL
);

ALTER TABLE UNIDADE ADD CONSTRAINT PK_UNIDADE PRIMARY KEY(UND_CODIGO);

INSERT INTO UNIDADE (UND_DESCRICAO, UND_SIGLA) VALUES ('CAIXA', 'CXA');
INSERT INTO UNIDADE (UND_DESCRICAO, UND_SIGLA) VALUES ('UNIDADE', 'UND');
INSERT INTO UNIDADE (UND_DESCRICAO, UND_SIGLA) VALUES ('KILO', 'KG');
INSERT INTO UNIDADE (UND_DESCRICAO, UND_SIGLA) VALUES ('METRO', 'MT');

-- PRODUTO
CREATE TABLE PRODUTO(
  PRO_CODIGO		SERIAL NOT NULL,
  PRO_C_PRINCIPAL	VARCHAR(20),
  PRO_STATUS		BOOLEAN,
  PRO_DESCRICAO		VARCHAR(200),
  PRO_DESC_REDUZIDO	VARCHAR(50),
  MAR_CODIGO		INTEGER,
  UND_CODIGO		INTEGER
);

ALTER TABLE PRODUTO ADD CONSTRAINT PK_PRODUTO PRIMARY KEY(PRO_CODIGO);
ALTER TABLE PRODUTO ADD CONSTRAINT FK_PRODUTO_MARCA FOREIGN KEY(MAR_CODIGO) REFERENCES MARCA(MAR_CODIGO);
ALTER TABLE PRODUTO ADD CONSTRAINT FK_PRODUTO_UNIDADE FOREIGN KEY(UND_CODIGO) REFERENCES UNIDADE(UND_CODIGO);

-- ESTOQUE
CREATE TABLE ESTOQUE(
  EST_CODIGO		SERIAL NOT NULL,
  PRO_CODIGO		INTEGER NOT NULL, -- PRODUTO
  EMP_CODIGO		INTEGER NOT NULL, -- EMPRESA
  EST_DATA		DATE NOT NULL,
  EST_HORA		TIME NOT NULL,
  EST_DOCUMENTO		VARCHAR(20),
  EST_SALDO		NUMERIC(15,5),
  EST_VLR_CUSTO		NUMERIC(15,5),
  EST_TIPO_MOV		CHAR,
  EST_QUANTIDADE	NUMERIC(15,5),
  EST_FLAG		BOOLEAN
);

ALTER TABLE ESTOQUE ADD CONSTRAINT PK_ESTOQUE PRIMARY KEY(EST_CODIGO);
ALTER TABLE ESTOQUE ADD CONSTRAINT FK_ESTOQUE_PRODUTO FOREIGN KEY(PRO_CODIGO) REFERENCES PRODUTO(PRO_CODIGO);
ALTER TABLE ESTOQUE ADD CONSTRAINT FK_ESTOQUE_EMPRESA FOREIGN KEY(EMP_CODIGO) REFERENCES EMPRESA(EMP_CODIGO);

